@@ -2,8 +2,9 @@ import logging
 import os
 import re
 import sys
+from collections import UserDict
 from copy import deepcopy
-from typing import Optional, Any, Dict, List, Union
+from typing import Any, Dict, List, Optional, Union
 
 import jsonpickle
 
@@ -11,8 +12,8 @@ from .common import LocalArtifact, S3Artifact
 from .config import config, s3_config
 from .context_syntax import GLOBAL_CONTEXT
 from .executor import Executor
-from .io import (PVC, ArgoVar, IfExpression, InputArtifact, InputParameter,
-                 OutputArtifact, OutputParameter)
+from .io import (PVC, ArgoVar, InputArtifact, InputParameter, OutputArtifact,
+                 OutputParameter)
 from .op_template import OPTemplate, PythonScriptOPTemplate, ShellOPTemplate
 from .python import Slices
 from .resource import Resource
@@ -32,54 +33,6 @@ except Exception:
 uploaded_python_packages = []
 
 
-class FutureLen:
-    '''
-    To solve the problem of delayed acquisition for length of output
-    parameters in the debug mode
-    '''
-
-    def __init__(self, par):
-        self.par = par
-
-    def __repr__(self):
-        return "len(%s)" % self.par
-
-    def get(self, context=None):
-        if context is not None and isinstance(self.par, InputParameter):
-            return len(context.inputs.parameters[self.par.name].value)
-        if isinstance(self.par, FutureRange):
-            return len(self.par.get(context))
-        return len(self.par.value)
-
-
-class FutureRange:
-    def __init__(self, *args):
-        self.args = args
-
-    def __repr__(self):
-        return "range(%s)" % ", ".join(map(str, self.args))
-
-    def get(self, context=None):
-        args = []
-        for i in self.args:
-            if isinstance(i, FutureLen):
-                args.append(i.get(context))
-            elif isinstance(i, (InputParameter, OutputParameter)):
-                args.append(i.value)
-            elif isinstance(i, IfExpression):
-                _if = render_expr(i._if, context)
-                if eval_expr(_if):
-                    args.append(int(eval(render_expr(i._then, context))))
-                else:
-                    args.append(int(eval(render_expr(i._else, context))))
-            elif isinstance(i, ArgoVar):
-                args.append(int(eval_expr(render_expr(str(i), context))))
-            else:
-                args.append(i)
-        args = tuple(args)
-        return list(range(*args))
-
-
 class ArgoRange(ArgoVar):
     def __init__(self, end, start=0, step=1):
         self.end = end
@@ -95,6 +48,67 @@ class ArgoRange(ArgoVar):
                          (start, end, step))
 
 
+class ObjectDict(UserDict):
+    def __init__(self, d=None):
+        super().__init__(d)
+
+    def __getattr__(self, key):
+        if key == "data":
+            return super().__getattr__(key)
+
+        value = self.data[key]
+        if isinstance(value, dict):
+            return ObjectDict(value)
+        return value
+
+    def __getitem__(self, key):
+        value = self.data[key]
+        if isinstance(value, dict):
+            return ObjectDict(value)
+        return value
+
+
+class Expression:
+    def __init__(self, expr):
+        self.expr = expr
+
+    def __repr__(self):
+        return self.expr
+
+    def eval(self, context):
+        from .dag import DAG
+        from .steps import Steps
+        inputs = ObjectDict()
+        inputs["parameters"] = {
+            k: v.value for k, v in context.inputs.parameters.items()
+        }
+        steps = ObjectDict()
+        if isinstance(context, Steps):
+            for step in sum([s if isinstance(s, list) else [s]
+                             for s in context], []):
+                steps[step.name] = {"outputs": {
+                    "parameters": {k: v.value for k, v in
+                                   step.outputs.parameters.items()
+                                   if hasattr(v, "value")},
+                }}
+        tasks = ObjectDict()
+        if isinstance(context, DAG):
+            for task in context:
+                tasks[task.name] = {"outputs": {
+                    "parameters": {k: v.value for k, v in
+                                   task.outputs.parameters.items()
+                                   if hasattr(v, "value")},
+                }}
+        variables = {"inputs": inputs, "steps": steps, "tasks": tasks}
+        return eval(self.expr, variables)
+
+
+def to_expr(var):
+    if isinstance(var, ArgoVar):
+        return var.expr
+    return str(var)
+
+
 def argo_range(
         *args,
 ) -> ArgoVar:
@@ -105,7 +119,7 @@ def argo_range(
     Each argument can be Argo parameter
     """
     if config["mode"] == "debug":
-        return FutureRange(*args)
+        return Expression("list(range(%s))" % ", ".join(map(to_expr, args)))
     start = 0
     step = 1
     if len(args) == 1:
@@ -206,7 +220,7 @@ def argo_len(
         param: the Argo parameter which is a list
     """
     if config["mode"] == "debug":
-        return FutureLen(param)
+        return Expression("len(%s)" % to_expr(param))
     return ArgoLen(param)
 
 
@@ -1094,10 +1108,7 @@ class Step:
                 self.inputs.parameters[k].source = v
                 continue
 
-            if v is None:
-                self.inputs.parameters[k].value = "null"
-            else:
-                self.inputs.parameters[k].value = v
+            self.inputs.parameters[k].value = v
 
     def set_artifacts(self, artifacts):
         for k, v in artifacts.items():
@@ -1164,7 +1175,53 @@ class Step:
                             value=v.template.inputs.parameters[
                                 "dflow_%s_urn" % v.name])
 
+    def render_by_executor(self, context=None):
+        if self.executor is not None:
+            assert isinstance(self.executor, Executor)
+            self.template = self.executor.render(self.template)
+            if hasattr(self.executor, "merge_sliced_step") and \
+                    self.executor.merge_sliced_step:
+                self.inputs.parameters["dflow_with_param"] = \
+                    InputParameter(value="")
+                self.inputs.parameters["dflow_sequence_start"] = \
+                    InputParameter(value=0)
+                self.inputs.parameters["dflow_sequence_end"] = \
+                    InputParameter(value=None)
+                self.inputs.parameters["dflow_sequence_count"] = \
+                    InputParameter(value=None)
+                self.inputs.parameters["dflow_sequence_format"] = \
+                    InputParameter(value="")
+                if self.with_param is not None:
+                    self.inputs.parameters["dflow_with_param"].value = \
+                        self.with_param
+                    self.with_param = None
+                if self.with_sequence is not None:
+                    start = self.with_sequence.start
+                    if start is None:
+                        start = 0
+                    end = self.with_sequence.end
+                    count = self.with_sequence.count
+                    format = self.with_sequence.format
+                    self.inputs.parameters["dflow_sequence_start"].value = \
+                        start
+                    self.inputs.parameters["dflow_sequence_end"].value = end
+                    self.inputs.parameters["dflow_sequence_count"].value = \
+                        count
+                    self.inputs.parameters["dflow_sequence_format"].value = \
+                        format
+                    self.with_sequence = None
+        elif context is not None:
+            self.template = context.render(self.template)
+
     def prepare_argo_arguments(self, context=None):
+        if isinstance(self.with_param, ArgoVar):
+            self.with_param = "{{=%s}}" % self.with_param.expr
+        elif self.with_param is not None and not isinstance(self.with_param,
+                                                            str):
+            self.with_param = jsonpickle.dumps(list(self.with_param))
+
+        self.render_by_executor(context)
+
         self.argo_parameters = []
         self.argo_artifacts = []
         for par in self.inputs.parameters.values():
@@ -1181,18 +1238,6 @@ class Step:
             else:
                 self.argo_artifacts.append(art.convert_to_argo())
 
-        if isinstance(self.with_param, ArgoVar):
-            self.with_param = "{{=%s}}" % self.with_param.expr
-        elif self.with_param is not None and not isinstance(self.with_param,
-                                                            str):
-            self.with_param = jsonpickle.dumps(list(self.with_param))
-
-        if self.executor is not None:
-            assert isinstance(self.executor, Executor)
-            self.template = self.executor.render(self.template)
-        elif context is not None:
-            self.template = context.render(self.template)
-
         if self.continue_on_num_success or self.continue_on_success_ratio is \
                 not None:
             if (isinstance(self.template, ShellOPTemplate)):
@@ -1226,9 +1271,7 @@ class Step:
 
     def run(self, context):
         self.phase = "Running"
-        if self.executor is not None:
-            assert isinstance(self.executor, Executor)
-            self.template = self.executor.render(self.template)
+        self.render_by_executor()
 
         import os
         from copy import copy
@@ -1246,10 +1289,8 @@ class Step:
         parameters = deepcopy(self.inputs.parameters)
         for name, par in parameters.items():
             value = par.value
-            if isinstance(value, FutureLen):
-                par.value = value.get(context)
-            elif isinstance(value, FutureRange):
-                par.value = value.get(context)
+            if isinstance(value, Expression):
+                par.value = value.eval(context)
             elif isinstance(value, (InputParameter, OutputParameter)):
                 par.value = get_var(value, context).value
             elif isinstance(value, ArgoVar):
@@ -1352,8 +1393,8 @@ class Step:
             return
 
         if self.with_param is not None or self.with_sequence is not None:
-            if isinstance(self.with_param, FutureRange):
-                item_list = self.with_param.get(context)
+            if isinstance(self.with_param, Expression):
+                item_list = self.with_param.eval(context)
             elif isinstance(self.with_param, (InputParameter,
                                               OutputParameter)):
                 item_list = self.with_param.value
@@ -1363,8 +1404,8 @@ class Step:
                 start = 0
                 if self.with_sequence.start is not None:
                     start = self.with_sequence.start
-                    if isinstance(start, FutureLen):
-                        start = start.get(context)
+                    if isinstance(start, Expression):
+                        start = start.eval(context)
                     elif isinstance(start, (InputParameter, OutputParameter)):
                         start = start.value
                     elif isinstance(start, ArgoVar):
@@ -1372,8 +1413,8 @@ class Step:
                             str(start), context)))
                 if self.with_sequence.count is not None:
                     count = self.with_sequence.count
-                    if isinstance(count, FutureLen):
-                        count = count.get(context)
+                    if isinstance(count, Expression):
+                        count = count.eval(context)
                     elif isinstance(count, (InputParameter, OutputParameter)):
                         count = count.value
                     elif isinstance(count, ArgoVar):
@@ -1382,8 +1423,8 @@ class Step:
                     sequence = list(range(start, start + count))
                 if self.with_sequence.end is not None:
                     end = self.with_sequence.end
-                    if isinstance(end, FutureLen):
-                        end = end.get(context)
+                    if isinstance(end, Expression):
+                        end = end.eval(context)
                     elif isinstance(end, (InputParameter, OutputParameter)):
                         end = end.value
                     elif isinstance(end, ArgoVar):
